function Fmatrix = ESIE0calcpropagatematrix(edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,...
    isthinhole,receivers,vispartedgesfromr,frequency,Rstart,isthinplaneedgepair,quadraturematrix_pos,quadraturematrix_weights,...
    rRvec,thetaRvec,zRvec)
    % ESIE0calcpropagatematrix - propagates the sound pressure at the edges to a receiver
% position.
%
% Input parameters:
%   edgepairlist
%   nedgeelems
%   bigmatrixstartnums
%   bigmatrixendnums
% 	edgelengthvec       A list [nedge,1] of the lenghts of each edge.
% 	closwedangvec       A list [nedge,1] of the close-wedge angle of each
%                       edge.
%   edgenvecs
%   edgestartcoords
%   edgeendcoords
%   isthinhole
% 	reftoshortlistE     A matrix, [nedges,nedges] with a pointer to the
% 	                    short lists that contain edge-to-edge data.
% 	re1sho              A short list, [nshort,1] of the cylindrical radii
%                       of each edge startpoint relative to all other
%                       edges.
% 	re2sho              A short list, [nshort,1] of the cylindrical radii
%                       of each edge endpoint relative to all other
%                       edges.
% 	thetae1sho          A short list, [nshort,1] of the theta angle
%                       of each edge startpoint relative to all other
%                       edges.
% 	thetae2sho          A short list, [nshort,1] of the theta angle
%                       of each edge endpoint relative to all other
%                       edges.
% 	ze1sho              A short list, [nshort,1] of the z-value
%                       of each edge startpoint relative to all other
%                       edges.
% 	ze2sho              A short list, [nshort,1] of the z-value
%                       of each edge endpoint relative to all other
%                       edges.
%   receivers             
%   vispartedgesfromr
%   frequency
%   Rstart
%   isthinplaneedgepair
%   quadraturematrix_pos
%   quadraturematrix_weights
%   edgerelatedcoordsysmatrices
%
%   FSAMP, CAIR, SHOWTEXT   Global parameters.
%
% Output parameters:
%   Fmatrix                
%
% Uses no other functions from the ESIE0toolbox
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by
%   the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along
%   with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson 2013-01-31 (svensson@iet.ntnu.no)
%
% Fmatrix = ESIE0calcpropagatematrix(edgedata,edgetoedgedata,edgepairlist,nedgeelems,bigmatrixstartnums,bigmatrixendnums,...
%     isthinhole,receivers,vispartedgesfromr,frequency,Rstart,isthinplaneedgepair,quadraturematrix_pos,quadraturematrix_weights,...
%     rRvec,thetaRvec,zRvec);

% 20121025 Added the isthinhole parameter
% 20130131 Cleaned up 
% 20130313 Reintroduced thetae2sho to handle non-convex geometries
% 20140207 Removed the factor 1/2 for alongplane radiation - moved to
%           calcedgeinteqmatrix!
% 20141201 Added the input parameter edgerelatedcoordsysmatrices
% 20141202 Replaced the input parameter edgerelatedcoordsysmatrices with
%          rRvec,thetaRvec,zRvec

global SHOWTEXT CAIR

% ----------------------------------------------------------------------------------------------
% Create the F-matrix by first creating a matrix showing which
% submatrices should be included.

nbig = bigmatrixendnums(end);
Fmatrix = zeros(1,nbig);
refto = edgetoedgedata.reftoshortlistE;

k = 2*pi*frequency/CAIR;

if SHOWTEXT >= 2
    disp(' ')
    disp(['      Building the F matrix of size (1,',int2str(nbig),')'])
end

n1previous = 0;
n2previous = 0;
edge2previous = 0;
edge1previous = 0;

alledgepairsthin = prod(double(isthinplaneedgepair));
noedgepairsthin  = prod(1-double(isthinplaneedgepair));

ivrelevantedgepairs = find( vispartedgesfromr(edgepairlist(:,1)) > 0 );

% for ii = 1:size(edgepairlist,1)    
for iicounter = 1:length(ivrelevantedgepairs)
    ii = ivrelevantedgepairs(iicounter);
    edge2 = edgepairlist(ii,1);    
    edge1 = edgepairlist(ii,2);    
    
    n1 = nedgeelems(edge1);
    n2 = nedgeelems(edge2);
%     len1 = edgelengthvec(edge1);
    len2 = edgedata.edgelengthvec(edge2);
    
    if alledgepairsthin == 1
        thinplaneboost = 2;
    else
        if noedgepairsthin == 1
            thinplaneboost = 1;
        else
            thinplaneboost = isthinplaneedgepair(ii)+1;
        end
    end
    
%     if vispartedgesfromr(edge2) > 0
        if SHOWTEXT >= 3
            disp(['      From edge ',int2str(edge1),' via edge ',int2str(edge2),' to R with thinplaneboost = ',int2str(thinplaneboost)])
            disp(['         Startcol in F-matrix   = (',int2str(bigmatrixstartnums(ii)),')'])
            disp(['         Endcol in F-matrix   = (',int2str(bigmatrixendnums(ii)),')'])
        end
        
        ny = pi/(2*pi-edgedata.closwedangvec(edge2));

        if n2 ~= n2previous 
            n2vec = quadraturematrix_pos(n2,1:n2);
            n2vec = n2vec(:);
            weightvec2 = quadraturematrix_weights(n2,1:n2);
            weightvec2 = weightvec2(:);
            
            if edge2 ~= edge2previous
                dzvec2 = weightvec2*edgedata.edgelengthvec(edge2);   
                ze2_re2 = (0 + n2vec*len2);
                edge2previous = edge2;
            end
            
        end
        
        if n1 ~= n1previous 
            n1vec = quadraturematrix_pos(n1,1:n1);
            n1vec = n1vec(:);
            weightvec1 = quadraturematrix_weights(n1,1:n1);
            weightvec1 = weightvec1(:);
            if edge1 ~= edge1previous
                dzvec1 = weightvec1*edgedata.edgelengthvec(edge1);
                edge1previous = edge1;
            end
            
        end 
        
        ze1_re2 = edgetoedgedata.ze1sho(refto(edge1,edge2)) + n1vec*( edgetoedgedata.ze2sho(refto(edge1,edge2))-edgetoedgedata.ze1sho(refto(edge1,edge2))   );
        re1_re2 = edgetoedgedata.re1sho(refto(edge1,edge2)) + n1vec*( edgetoedgedata.re2sho(refto(edge1,edge2))-edgetoedgedata.re1sho(refto(edge1,edge2))   );

        edgecoords = [edgedata.edgestartcoords(edge2,:);edgedata.edgeendcoords(edge2,:)];
        rR = rRvec(edge2);
        thetaout = thetaRvec(edge2);
        zR = zRvec(edge2);
        
        thetain = edgetoedgedata.thetae1sho(refto(edge1,edge2));
        thetain_end = edgetoedgedata.thetae2sho(refto(edge1,edge2));
        if (abs(thetain) < 1e-10 || abs(thetain-(2*pi-edgedata.closwedangvec(edge2))) < 1e-10) && abs(thetain-thetain_end) < 1e-10
            acrossface_in = 1;
        else
            acrossface_in = 0;
            
            xedge1start = edgetoedgedata.re1sho(refto(edge1,edge2))*sin(edgetoedgedata.thetae1sho(refto(edge1,edge2)));
            xedge1end   = edgetoedgedata.re2sho(refto(edge1,edge2))*sin(edgetoedgedata.thetae2sho(refto(edge1,edge2)));
            xe1_re2 = xedge1start + n1vec.*(xedge1end-xedge1start);
            thetae1_re2 = asin(xe1_re2./re1_re2);            
        end        

        % Build vertical [n2,n1] matrices and expand them horizontally
        if n2 ~= n2previous || n1 ~= n1previous
            n2vertmat = uint8(1:n2);
            n2vertmat = reshape(repmat(n2vertmat,n1,1),n1*n2,1);           
            n2previous = n2;
            n1vertmat = uint8(1:n1).';
            n1vertmat = repmat(n1vertmat,n2,1);    
            n1previous = n1;
        end
        
        ldist = (zR - ze2_re2(n2vertmat)).^2   + rR.^2;
        ldist = sqrt(ldist);
        
        mdist = (ze1_re2(n1vertmat) - ze2_re2(n2vertmat)).^2   + re1_re2(n1vertmat).^2;
        mdist = sqrt(mdist);

        dz_expjkmloverml = exp(-j*k*(ldist+mdist-Rstart))./ldist./mdist.*dzvec2(n2vertmat).*dzvec1(n1vertmat);       

        ch = ((ze2_re2(n2vertmat) - ze1_re2(n1vertmat)).*(ze2_re2(n2vertmat) - zR) + mdist.*ldist)./re1_re2(n1vertmat)./rR;
        ch = ( real(sqrt( ch.^2-1)) + ch ).^ny;
        ch = ( ch + 1./ch)/2;

        % 28 March 2011: thetain is always zero (or = theta_wedge) so terms
        % 1 and 3 will always be identical; and terms 2 and 4.

        if isthinhole == 0
            if acrossface_in == 1            
                beta = 2* ( sin(ny*(pi + thetain + thetaout  ))./(ch - cos(ny*(pi + thetain + thetaout   ))) + ...
                     sin(ny*(pi + thetain - thetaout  ))./(ch - cos(ny*(pi + thetain - thetaout   ))));  
                 
            else
                beta =  sin(ny*(pi + thetae1_re2(n1vertmat) + thetaout  ))./(ch - cos(ny*(pi + thetae1_re2(n1vertmat) + thetaout   ))) + ...
                        sin(ny*(pi + thetae1_re2(n1vertmat) - thetaout  ))./(ch - cos(ny*(pi + thetae1_re2(n1vertmat) - thetaout   ))) + ...                                         
                        sin(ny*(pi - thetae1_re2(n1vertmat) + thetaout  ))./(ch - cos(ny*(pi - thetae1_re2(n1vertmat) + thetaout   ))) + ...
                        sin(ny*(pi - thetae1_re2(n1vertmat) - thetaout  ))./(ch - cos(ny*(pi - thetae1_re2(n1vertmat) - thetaout   )));
            end            
 
            % Change 7 Feb. 2014
%              Fsub =             -(2-acrossface_in)*ny/4/2/pi*beta.*dz_expjkmloverml*thinplaneboost; disp('Prop OLD')
            Fsub = -ny/4/2/pi*beta.*dz_expjkmloverml*thinplaneboost;  % disp('Prop NEW')
%             acrossface_in
            

        else % ny must be 1/2 to end up here
%            dbetadthetaS = -4*ch*cos(thetaout/2).*(cos(thetaout) + 2*ch.^2 - 3)./(cos(thetaout)^2 - 2*ch.^2 + 1)./(1+j*k*mdist); 
%            Fsub = -1/8/2/pi*dbetadthetaS.*dz_expjkmloverml*thinplaneboost;   % Version 0
            dbetadthetaS = -4*ch*cos(thetaout/2).*(cos(thetaout) + 2*ch.^2 - 3)./(cos(thetaout)^2 - 2*ch.^2 + 1); 
            Fsub = -1/8/2/pi*dbetadthetaS.*dz_expjkmloverml./mdist*thinplaneboost;   % Version 1
        end
        
        if SHOWTEXT >= 3
            fivec = pi + thetain*[1 1 -1 -1] + thetaout*[1 -1 1 -1];
            absnyfivec = abs(ny*fivec); 
            useserialexp1 = absnyfivec < 0.01;
            useserialexp2 = abs(absnyfivec - 2*pi) < 0.01;
            useserialexp = useserialexp1 | useserialexp2;
            if any(useserialexp)
                disp('WARNING! Analytical approximation should be used')
            end
            plot(abs(Fsub))
            
        end
        
        Fmatrix(bigmatrixstartnums(ii):bigmatrixendnums(ii)) =     Fmatrix(bigmatrixstartnums(ii):bigmatrixendnums(ii)) + Fsub.';
%     end
end

