function [Qfirstterm,nedgeelems] = ESIE0integ_souterm(planeeqs,edgedata,edgetoedgedata,edgepairlist,...
    nedgeelems,bigmatrixstartnums,bigmatrixendnums,inteq_ngauss,inteq_discretizationtype,...
    sources,vispartedgesfroms,vispartedgesfromIS,frequency,gaussvectors,rSvec,thetaSvec,zSvec)
% ESIE0integ_souterm - Does the first step of the ED int eq: compute the 
% first term of the edge source signals.
%
% Input parameters:
%   edgepairlist
%   nedgeelems
%   bigmatrixstartnums
%   bigmatrixendnums
% 	edgelengthvec       A list [nedge,1] of the lenghts of each edge.
% 	closwedangvec       A list [nedge,1] of the close-wedge angle of each
%                       edge.
%   edgenvecs
%   edgestartcoords
%   edgeendcoords
% 	inteq_ngauss        A value with the number of gauss points
%   inteq_discretizationtype    0 (midpoint method) or 2 (Gauss-Legendre)
% 	reftoshortlistE     A matrix, [nedges,nedges] with a pointer to the
% 	                    short lists that contain edge-to-edge data.
% 	re1sho              A short list, [nshort,1] of the cylindrical radii
%                       of each edge startpoint relative to all other
%                       edges.
% 	re2sho              A short list, [nshort,1] of the cylindrical radii
%                       of each edge endpoint relative to all other
%                       edges.
% 	thetae1sho          A short list, [nshort,1] of the theta angle
%                       of each edge startpoint relative to all other
%                       edges.
% 	ze1sho              A short list, [nshort,1] of the z-value
%                       of each edge startpoint relative to all other
%                       edges.
% 	ze2sho              A short list, [nshort,1] of the z-value
%                       of each edge endpoint relative to all other
%                       edges.
%   sources          
%   vispartedgesfroms
%   vispartedgesfromIS
%   planeeqs
%   frequency
%   gaussvectors
%   rSvec,thetaSvec,zSvec
%   CAIR, SHOWTEXT   Global parameters.
%
% Output parameters:
%   Qfirstterm                
%   nedgeelems
%
% Uses functions ESIE0distelements ESIE0findis 
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by
%   the Free Software     
%   Foundation, either version 3 of the License, or (at your option) afny later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson 2014-12-01 (svensson@iet.ntnu.no)
%
% [Qfirstterm,nedgeelems] = ESIE0integ_souterm(edgepairlist,...
%    nedgeelems,bigmatrixstartnums,bigmatrixendnums,edgelengthvec,closwedangvec,edgenvecs,edgestartcoords,edgeendcoords,...
%    inteq_ngauss,inteq_discretizationtype,reftoshortlistE,re1sho,re2sho,thetae1sho,thetae2sho,ze1sho,ze2sho,...
%    sources,vispartedgesfroms,frequency,gaussvectors,sourcetype,sourcedata,rSvec,thetaSvec,zSvec);

% 20120217 Functioning version
% 20121025 Added the isthinhole output parameter
% 20130129 Cleaned up thoroughly
% 20130130 Removed the isthinhole output parameter
% 20140207 Introduced the factor 1/2 scaling (and removed it from the
% propagation). Also introduced the input parameter vispartedgesfromIS
% 20140923 Fixed a bug for concave geometries and skewed edge pairs
% 20141201 Added the input parameter edgerelatedcoordsysmatrices
% 20141202 Replaced the input parameter edgerelatedcoordsysmatrices with
%          rSvec,thetaSvec,zSvec

global SHOWTEXT CAIR

isthinhole = 0;
discretizationtype = inteq_discretizationtype;

symmetrycompression = 1;
if SHOWTEXT >= 2
    disp(['   ESIE0integ_souterm: symmetrycompression = ',int2str(symmetrycompression)])
end

% ----------------------------------------------------------------------------------------------
% Create the G-matrix

G = zeros(bigmatrixendnums(end),1);
refto = edgetoedgedata.reftoshortlistE;

k = 2*pi*frequency/CAIR;

if SHOWTEXT >= 2
    disp(' ')
    disp(['      Building the G matrix of size (',int2str(bigmatrixendnums(end)),',1)'])
end

% ----------------------------------------------------------------------------------------------
% Go through all the edge-pair combos

nyvec = pi./(2*pi-edgedata.closwedangvec);

n2previous = 0;
n3previous = 0;

for ii = 1:size(edgepairlist,1)    
    edge2 = edgepairlist(ii,2);
    
    if vispartedgesfroms(edge2) ~= 0
        edge3 = edgepairlist(ii,1);
        if SHOWTEXT >= 3
            disp(' ')
            disp(['      From S via edge ',int2str(edge2),' to edge ',int2str(edge3)])
            disp(['         Startrow in G-matrix   = (',int2str(bigmatrixstartnums(ii)),')'])
            disp(['         Endrow in G-matrix   = (',int2str(bigmatrixendnums(ii)),')'])
            nfrom = nedgeelems(edge2);
            nto   = nedgeelems(edge3);
        end
        
        ny = nyvec(edge2);
        len2 = edgedata.edgelengthvec(edge2);
        n2 = nedgeelems(edge2);
        n3 = nedgeelems(edge3);
                
        if size(gaussvectors,1) == n2
            n2vec = gaussvectors(:,1);
        else
           [n2vec,~] = ESIE0distelements(n2,discretizationtype);
        end
        if size(gaussvectors,1) == n3
            n3vec = gaussvectors(:,1);
        else
           [n3vec,~] = ESIE0distelements(n3,discretizationtype);
        end
        
        % FOund bug 23 Sep. 2014: did not take into account that the other
        % end of the edge might have a different theta value.
        thetaout = edgetoedgedata.thetae1sho(refto(edge3,edge2));
        thetaout_otherend = edgetoedgedata.thetae2sho(refto(edge3,edge2));
        
%         if (abs(thetaout) < 1e-10 || abs(thetaout-(2*pi-closwedangvec(edge2))) < 1e-10)
        if (abs(thetaout) < 1e-10 || abs(thetaout-(2*pi-edgedata.closwedangvec(edge2))) < 1e-10) && abs(thetaout-thetaout_otherend)<1e-10
            acrossface_out = 1;
        else
            acrossface_out = 0;
        end     
        if SHOWTEXT >= 3
            disp(['         Acrossface_out = ',int2str(acrossface_out)])
        end
        
        edgecoords = [edgedata.edgestartcoords(edge2,:);edgedata.edgeendcoords(edge2,:)];
        rS = rSvec(edge2);
        thetaS = thetaSvec(edge2);
        zS = zSvec(edge2);

        % Build vertical [n3,n2] matrices and expand them horizontally
        if n3 ~= n3previous || n2 ~= n2previous 
            n3vertmat = uint8(1:n3);
            n3vertmat = reshape(repmat(n3vertmat,n2,1),n2*n3,1);
            n3previous = n3;
            n2vertmat = uint8(1:n2).';
            n2vertmat = repmat(n2vertmat,n3,1);        
            n2previous = n2;
        end
        
        ze2_re2 = n2vec*len2;
        ze3_re2 = edgetoedgedata.ze1sho(refto(edge3,edge2)) + n3vec*( edgetoedgedata.ze2sho(refto(edge3,edge2))-edgetoedgedata.ze1sho(refto(edge3,edge2))   );
        re3_re2 = edgetoedgedata.re1sho(refto(edge3,edge2)) + n3vec*( edgetoedgedata.re2sho(refto(edge3,edge2))-edgetoedgedata.re1sho(refto(edge3,edge2))   );

        ldist = (ze3_re2(n3vertmat) - ze2_re2(n2vertmat)).^2   + re3_re2(n3vertmat).^2;
        ldist = sqrt(ldist);

        mdist = (zS - ze2_re2(n2vertmat)).^2   + rS.^2;
        mdist = sqrt(mdist);

%         dz_expjkmoverm = exp(-j*k*mdist)./mdist.*dzvec(n2vertmat);
        dz_expjkmoverm = exp(-j*k*mdist)./mdist;

        ch = ((ze2_re2(n2vertmat) - zS).*(ze2_re2(n2vertmat) - ze3_re2(n3vertmat)) + mdist.*ldist)./rS./re3_re2(n3vertmat);
        ch = ( real(sqrt( ch.^2-1)) + ch ).^ny;
        ch = ( ch + 1./ch)/2;

% % %         if isthinhole == 0
            if thetaout == 0
                beta = 2*( sin(ny*(pi + thetaS  ))./(ch - cos(ny*(pi + thetaS   ))) + ...
                           sin(ny*(pi - thetaS  ))./(ch - cos(ny*(pi - thetaS   ))));            
            else
                beta = ( sin(ny*(pi + thetaS + thetaout  ))./(ch - cos(ny*(pi + thetaS + thetaout   ))) + ...
                         sin(ny*(pi + thetaS - thetaout  ))./(ch - cos(ny*(pi + thetaS - thetaout   ))) + ...
                         sin(ny*(pi - thetaS + thetaout  ))./(ch - cos(ny*(pi - thetaS + thetaout   ))) + ...
                         sin(ny*(pi - thetaS - thetaout  ))./(ch - cos(ny*(pi - thetaS - thetaout   ))));           
            end
          Gsub = -(2-acrossface_out)*ny/4/pi*beta.*dz_expjkmoverm;
       
        G(bigmatrixstartnums(ii):bigmatrixendnums(ii)) =     G(bigmatrixstartnums(ii):bigmatrixendnums(ii)) + Gsub;
        
        ivfindIS = find(vispartedgesfromIS(:,edge2));
        if ~isempty(ivfindIS)
            for kk = ivfindIS
            
               disp('HERE WE SHOULD ADD SOMETHING!') 
               disp(['From S to plane ',int2str(kk),' to edge ',int2str(edge2),' in the direction of ',int2str(edge3)]) 
               
               [xis] = ESIE0findis(sources,kk,planeeqs,1,[1 1 1]);               
               
            end   
        end
        
        
    end
    
end

Qs = 1;
Qfirstterm = G*Qs;
