function ESIE0main(EDsetupfile)
% ESIE0main - Calculates the specular and edge diffraction TFs for convex, rigid scattering
% objects, using the image source method and the ESIE0. 
%
% Input parameters:
%	EDsetupfile 	(optional)	This file contains all parameter settings for the calculations.
%					If no filename is given, a file open window will appear and the
%					desired EDsetupfile can be chosen.
%                   The file could either be a .m file (the "EDsetupfile")
%                   or a .mat file (the "setupmatfile").
% Input parameters in the EDsetupfile:
%	FSAMP, CAIR, RHOAIR, SHOWTEXT
%						(global)	The sampling frequency, speed of sound, density of air
%									and the control of how much text should
%									be plotted (SHOWTEXT could be 0,1,2,3,4).
%   SUPPRESSFILES (global, optional) If this variable is introduced in the
%                       setupfile as a global variable, with the value 1, then the files
%                       'xxx_ISEStree.mat' will not be generated. Instead values are passed
%                       directly.
%	Filepath			The path to the folder where all output data files will be stored.
%	Filestem			The first part of the output data files will be given this name
%	CADfile			    The name of the .CAD file where the geometry is given, including path
%   open_or_closed_model      Specifies whether the geometrical model is open or closed:
%                       'o'     Open model (OK to specify 'open'; only first char. is tested)
%                       'c'     Closed model (OK to specify 'closed'; only first char. is tested)
%   int_or_ext_model    Specifies whether the geometrical model is interior or exterior:
%                       'i'     Interior model (NB! An interior model must be closed, 
%                               otherwise it should be called an exterior model)
%                               (OK to specify 'interior'; only first char. is tested)
%                       'e'     Exterior model (OK to specify 'exterior'; only first char. is tested)
%	directsound		0 or 1     Specifies if the direct sound should be included or not
%                              It is necessary to set directsound=0 if the source and receiver
%                              are in the same position.
%	specorder		The maximum order of specular reflections. (To get all possible combinations,
%                   specorder and difforder should be given the same value). Note that in the present
%                   version of ESIE0toolbox, only convex scatterers are handled, so specular 
%                   refelctions of higher order than 1 are impossible.
%	difforder		The maximum order of diffraction components. (difforder can never be higher
%                   than specorder, i.e., if you want difforder = 2, you must set specorder to 2 or higher)
%   nedgesubs       All edges are subdivided very coarsely into edge segments, for visibility checks.
%                   The value of nedgesubs states how many such segments each edge will be divided into.
%                   The minimum value is 2. NB! All edges will be divided into equally many segments
%                   in order to employ efficient Matlab vector-based processing.
%                   Note that in the present version of ESIE0toolbox, this
%                   parameter is not used.
%	sources			A vector of the source coordinates, size [nsources,3], or source names/numbers,
%                   size [nsources,1] or [nsources,2] (if they should be taken from the CADfile.)
%	receivers		A vector of the receiver coordinates or receiver numbers (if they should be taken
%					from the CADfile. Same syntax as for sources.
%
% Optional input parameters in the EDsetupfile:
%	cadgeofile	    If this parameter is specified, this file will be read rather than the CADfile.
%                   Saves some time for big geometries.
%	eddatafile	    If this parameter is specified, the function ESIE0edgeo is not called. Saves
%					some time.
%	srdatafile	    If this parameter is specified, the function ESIE0srgeo is not called. Saves
%					some time.
%   restartnumber   If this parameter is specified, the calculations will start at receiver number
%                   restartnumber, rather than number 1. This is useful if a calculation for many
%                   receivers can not be completed in one run.
%   stopnumber      If this parameter is specified, the calculations will stop for receiver number
%                   stopnumber.
%   saveindividualdiffirs   A vector with two values, 0 or 1.
%					If the first value is given the value 1, the diffraction
%                   IR will have one column for each order. Default is that
%                   all orders are summed into a single irdiff.
%					If the second value is given the value 1, all individual diffraction irs
%					will be saved on individual lines in a large matrix called 'alldiffirs'.
%				 	This matrix has only single precision.
%					Default value: [0 0]
%   doaddsources    0 or 1, (default value 0) indicating whether:
%                       0: every individual S-to-R IR should be saved
%                       1: a single IR is saved for each R, which is
%                       constructed by first computing all individual
%                       S-to-R IRs, saving them in a temp file, and then
%                       add IRs from all sources together. This way a
%                       loudspeaker element can be simulated by a
%                       distribution of point sources.
%   dobackscatter   0 or 1, (default value 0), indicating whether:
%                       0: every individual S-to-R IR should be saved
%                       1: only S1 to R1, S2 to R2, S3 to R3 combos
%                       are calculated.
%   nSbatches,nRbatches
%                   (optional) An integer >= 1 which defines how many chunks the number of sources/receivers
%                   will be divided into. Default = 1. nSbatches/nRbatches = 1 means that all sources/receivers are 
%                   treated simultaneously. nSbatches/nRbatches = 2 means that the list of sources/receivers
%                   is divided into two chunks etc. It is fastest with nSbatches/nRbatches = 1 but it also
%                   requires the largest amount of memory. (For large problems, there might even be no time
%                   penalty in choosing nRbatches = the number of receivers!)
%   soulist_for_findpaths/reclist_for_findpaths
%                   (optional) A list of integers that specifies which of the sources/receivers that
%                   should be run for the "find paths" stage. Default = [1:nsources]/[1:nreceivers].
%                   This is useful for huge problems where one might want to stop (with CTRL-C) and restart calculations.
%   soulist_for_calcirs/reclist_for_calcirs
%                   (optional) A list of integers that specifies which of the sources/receivers that
%                   should be run for the "calc irs" stage. Default = [1:nsources]/[1:nreceivers].
%                   This is useful for huge problems where one might want to stop (with CTRL-C) and restart calculations.
%	firstskipcorner (optional) All edges including at least one corner with this number or
%             		higher will be excluded from the calculations. Default: 1e6
%	Rstart 			(optional) The distance that should correspond to the distance zero for
%             		all TFs, which affects the Default = 0.
%
% Output parameters
%   Several results .mat files are generated, with the same filestem as the setup file (XXX).
%   XXX_cadgeo      Contains the information in the cad file. Generated by ESIE0readcad
%   XXX_eddata      Contains all edge-related data that are generated from the corners and 
%                   planes data in the cad file. Generated by ESIE0edgeox and ESIE0ed2geox
%   XXX_sdata       Contains all source-related data. Generated by ESIE0SorRgeo
%   XXX_rdata       Contains all receiver-related data. Generated by ESIE0SorRgeo
%   XXX_tf          Contains the resulting sound pressure in the form of direct sound,
%                   specular reflections, and first-order diffraction for all
%                   source-receiver-frequency combinations, stored as
%                       tfdirect, tfgeom, tfdiff
%                   Generated by ESIE0maketfs
%   XXX_tfinteq     Contains the resulting sound pressure representing
%                   second- and higher-order diffraction, stored as
%                       tfinteqdiff
%                   Generated by ESIE0integralequation
%   XXX_log         Contains a number of timing values, for various parts
%                   of the computations.
%
% Note that the transfer functions are scaled such that the direct sound gets an amplitude of
% 1/distance. This means that the transfer functions represent a system with the sound pressure as
% output signal and an input signal = RHOAIR*Volumeacc/(4*pi) where Volumeacc is
% the volume acceleration of the monopole source.
%
% Uses functions ESIE0readcad ESIE0readsetup ESIE0edgeox ESIE0SorRgeo
% ESIE0ed2geox ESIE0findISEStree ESIE0findpathsISESx ESIE0maketfs
% ESIE0integ_e2estructure ESIE0integralequation
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson (svensson@iet.ntnu.no) 28 November 2014
%
% ESIE0main(EDsetupfile);

% 20121025 Added the isthinhole to the ESIE0integ_souterm function
% 28 Sep. 2014 Added the input parameter listofcornerstoskip to the call of
%              the ESIE0edgeox function.
% 28 Nov. 2014 Tried to trim the amount of file saving.
% 31 March 2015 Added '' in file saving and loading functions, to permit
%               file/directory names with space.
% 31 March 2015 Cleared up the help text and the timing

global FSAMP CAIR RHOAIR SHOWTEXT JJ JJnumbofchars
global POTENTIALISES ISCOORDS IVNDIFFMATRIX
global IVNSPECMATRIX ORIGINSFROM ISESVISIBILITY REFLORDER

FSAMP = []; CAIR = []; RHOAIR = [];  SHOWTEXT = [];

%--------------------------------------------------------------
% First run the setup m file, just to get access to the CAD-file name
% so that the CAD-file can be converted into a cadgeofile.
% (This conversion must be done before the setup m-file is converted
% to a setup mat-file since the source and receiver coordinates should
% be specified in the setupmatfile, and the sources and receivers might
% have to be read from the CAD-file).
%
% First we make a little check of a common mistake: check if the paths
% seem to be for the correct computer type.
% The EDsetupfile-path does not need to be checked if a file window is
% presented. The CAD-file path always needs to be checked.

% We check if the type of setupfile:
%   .m    setupfiletype = 1;
%   .mat  setupfiletype = 2;

setupfiletype = 1;

if nargin == 0,
	[EDsetupfile,EDsetupfilepath] = uigetfile('*.m','Please select the EDsetupfile');
    [temp1,EDsetupfile,temp2] = fileparts([EDsetupfilepath,EDsetupfile]);

    EDsetupfile = [EDsetupfilepath,EDsetupfile];
    
	if ~isstr(EDsetupfile) | isempty(EDsetupfile),
		return
	end
else
    [EDsetupfilepath,tempfilestem,fileext] = fileparts(EDsetupfile);
    EDsetupfilepath = [EDsetupfilepath,filesep];
    if length(fileext) == 4
        if fileext == '.mat'
            setupfiletype = 2; 
        end
    end
end

eval(['cd ''',EDsetupfilepath,''''])

if setupfiletype == 1
    comptype = computer;
    if length(comptype) == 3,
        comptype = [comptype,'P'];
    end
    if comptype(1:4) == 'MACI', 
        [temp1,tempfilestem,temp2] = fileparts(EDsetupfile);
        eval([tempfilestem])        
    else    
        eval(['run ''',EDsetupfile,''''])
    end
else
    eval(['load ''',EDsetupfile,''''])
end

if SHOWTEXT >= 1,
	disp(' ')
	disp('####################################################################')
	disp('#')
	disp('#  ESIE0main, version 29 May 2015')
	disp('#')
    disp(' ')
    disp(['   Using the setupfile: ',EDsetupfile])
    disp(' ')
end

% #########################################################################
% #########################################################################
%   
%      Geometry pre-calculations
%   
% #########################################################################

%---------------------------------------------------------------------------------------------
% Convert the CAD-file into a mat file - or read the cadgeofile if it is specified

t00 = clock;

if SHOWTEXT >= 1,
	disp(' ')
	disp('####################################################################')
	disp('#')
	disp('#  Geometry pre-calculations')
	disp(' ')
end

if exist('cadgeofile') ~= 1,
    desiredname = [Filepath,Filestem,'_cadgeo'];
    if SHOWTEXT >= 2,
		disp(['   Reading CAD-file: ',CADfile,' and converting it to a cadgeofile: ',desiredname])
    end
    [planedata,cadgeofile] = ESIE0readcad(CADfile,desiredname);
else
	if SHOWTEXT >= 2,
		disp(['   Using existing cadgeofile: ',cadgeofile])
        eval(['load ',cadgeofile])
	end
end

%--------------------------------------------------------------
% Convert the EDsetupfile into a setupmatfile. If sources and receivers should
% be read in the CADfile, add these to the setupmatfile.
%
% This is needed only if the setupfile was of type 1, that is, a .m file.

if setupfiletype == 1
    setupmatfile = ESIE0readsetup(EDsetupfile,cadgeofile);

    eval(['load ''',setupmatfile,''''])
end

tcadgeo = etime(clock,t00);

%--------------------------------------------------------------
% Derive the edge parameters.

t00 = clock;

if calcpaths == 1,
    
    if exist('eddatafile') ~= 1,
	    desiredfile = [Filepath,Filestem,'_eddata'];
		if SHOWTEXT >= 2,
			disp(['   Calculating edges and creating an eddatafile: ',desiredfile])
		end
		[edgedata,planedata,eddatafile] = ESIE0edgeox(planedata,desiredfile,lower(open_or_closed_model(1)),lower(int_or_ext_model(1)),...
            specorder,difforder,nedgesubs,firstskipcorner,listofcornerstoskip,planeseesplanestrategy);
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing eddatafile: ',eddatafile])
		end
	end
else
	if exist('eddatafile') ~= 1,
	    desiredfile = [Filepath,Filestem,'_eddata'];
    	eddatafile = desiredfile;
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing eddatafile: ',eddatafile])
		end
	end
end

tedgeo = etime(clock,t00);

nsources = size(sources,1);
nreceivers = size(receivers,1);
nfrequencies = length(frequencies);

doSbatches = 0;    
doRbatches = 0;    

%---------------------------------------------------------------------------------------------
% Find out the S and R related parameters, make obstruction checks etc by calling 
% ESIE0Sgeo/ESIE0Rgeo or load existing sdata/rdata files.

t00 = clock;

if calcpaths == 1,
	if exist('sdatafile') ~= 1,
        desiredname = [Filepath,Filestem,'_sdata'];
        if SHOWTEXT >= 2,
            disp(['   Calculating S parameters and creating an sdatafile: ',desiredname])
        end

        [Sdata,sdatafile] = ESIE0SorRgeo(planedata,edgedata,desiredname,sources,'S',difforder,nedgesubs,int_or_ext_model);
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing sdatafile: ',sdatafile])
		end
	end
else
	if exist('sdatafile') ~= 1,
	    desiredname = [Filepath,Filestem,'_sdata'];
    	sdatafile = desiredname;
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing sdatafile: ',sdatafile])
		end
	end
end

tsgeo = etime(clock,t00);


%------------------------------------
% We check that there is not a small geometrical mistake which makes the
% source be behind all planes - but only if the user has set SHOWTEXT.

if SHOWTEXT >= 2,
    eval(['load ',sdatafile])
    if size(visplanesfroms,2) == 1,
        if ~any(visplanesfroms),
            disp(' ')
            disp('WARNING!!! The source(s) can not see a single plane!')
            disp('           Check if this is correct! ')
            disp('           A common cause is that the source is placed very close to a plane,')
            disp('           but behind it. (CR => continue calculations)')
            disp(' ')
            pause
        end                
    else
        iv = find(sum(visplanesfroms) == 0);
        if ~isempty(iv),
            disp(' ')
            disp('WARNING!!! Some of the source(s) can not see a single plane!')
            disp('           These are the sources number:')
           disp(['           ',int2str(iv(:).')])
            disp('           Check if this is correct! ')
            disp('           A common cause is that the source is placed very close to a plane,')
            disp('           but behind it. (CR => continue calculations)')
            disp(' ')
        end
    end
end

t00 = clock;

if calcpaths == 1,
	if exist('rdatafile') ~= 1,
        desiredname = [Filepath,Filestem,'_rdata'];
        if SHOWTEXT >= 2,
            disp(['   Calculating R parameters and creating an rdatafile: ',desiredname])
        end
        [Rdata,rdatafile] = ESIE0SorRgeo(planedata,edgedata,desiredname,receivers,'R',difforder,nedgesubs,int_or_ext_model);
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing rdatafile: ',rdatafile])
		end
	end
else
	if exist('rdatafile') ~= 1,
	    desiredname = [Filepath,Filestem,'_rdata'];
    	rdatafile = desiredname;
	else
		if SHOWTEXT >= 2,
			disp(['   Using existing rdatafile: ',rdatafile])
		end
	end
end

trgeo = etime(clock,t00);

%------------------------------------
% We check that there is not a small geometrical mistake which makes the
% receiver(s) be behind all planes - but only if the user has set SHOWTEXT.

if SHOWTEXT >= 2,
    eval(['load ',rdatafile])
    if size(visplanesfromr,2) == 1,
        if ~any(visplanesfromr),
            disp(' ')
            disp('WARNING!!! The receiver can not see a single plane!')
            disp('           Check if this is correct! ')
            disp('           A common cause is that the receiver is placed very close to a plane,')
            disp('           but behind it. (CR => continue calculations)')
            disp(' ')
            pause
        end                
    else
        iv = find(sum(visplanesfromr) == 0);
        if ~isempty(iv),
            disp(' ')
            disp('WARNING!!! Some of the receiver(s) can not see a single plane!')
            disp('           These are the receiver numbers:')
           disp(['           ',int2str(iv(:).')])
            disp('           Check if this is correct! ')
            disp('           A common cause is that a receiver is placed very close to a plane,')
            disp('           but behind it. (CR => continue calculations)')
            disp(' ')
        end
    end
end

%---------------------------------------------------------------------------------------------
% The edgeseesedge test is run separately, for the cases where difforder >= 2. 
% The data is stored in the eddata file again.

t00 = clock;

if difforder >= 2 & calcpaths == 1,
% if difforder >= 1 & calcpaths == 1,
    if exist('eddata2file') ~= 1,
	    desiredfile = [Filepath,Filestem,'_eddata'];
    	if SHOWTEXT >= 2,
		    disp(['   Adding edge-to-edge visibility to the eddatafile: ',eddatafile])
    	end
        if exist('ndiff2batches') ~= 1,
            ndiff2batches = 1;    
        end
    	[edgetoedgedata,eddatafile] = ESIE0ed2geox(edgedata,planedata,eddatafile,sdatafile,rdatafile,doSbatches,doRbatches,specorder,difforder,nedgesubs,ndiff2batches,int_or_ext_model);    

	else
		if SHOWTEXT >= 2,
			disp(['   Using existing eddata2file: ',eddata2file])
		end
	end
    
end

ted2geo = etime(clock,t00);

%---------------------------------------------------------------------------------------------
% Make a big loop running through the source and receiver coordinates.

t00 = clock;

% eval(['load ',eddatafile])
% clear cornerinfrontofplane planeseesplane edgeseesplane

nplanes     = size(planedata.planecorners,1);
nedges      = size(edgedata.edgecorners,1);

if exist('calcpaths')~=1,
	calcpaths = 1;
end
if exist('calcirs')~=1,
	calcirs = 1;
end

% Display receiver numbers with an interval that depends on the 
% value of SHOWTEXT.

if SHOWTEXT == 1,
    idisp = ceil(nreceivers/20);
elseif SHOWTEXT > 1,
    idisp = 1;
end

% We construct the string version of counters and let them be global
% variables so we don't have to call int2str all the time

ncountersneeded = max(max([specorder difforder nplanes nedges nreceivers]))*2;
if ncountersneeded < 500
    ncountersneeded = 500;
end
if ncountersneeded < 10,
    JJ = setstr(32*ones(ncountersneeded,1));
elseif ncountersneeded < 100
    JJ = setstr(32*ones(ncountersneeded,2));
else
    JJ = setstr(32*ones(ncountersneeded,3));
end
for jj=1:ncountersneeded,
   jjstr = int2str(jj);
   JJ(jj,1:length(jjstr)) = jjstr;
end
[n1,n2] = size(JJ);
JJnumbofchars = ones(n1,1);
if n1>9,
    JJnumbofchars(10:n1) = JJnumbofchars(10:n1)+1;
    if n1 > 99,
        JJnumbofchars(100:n1) = JJnumbofchars(100:n1)+1;    
        if n1 > 999,
            JJnumbofchars(1000:n1) = JJnumbofchars(1000:n1)+1;    
        end
    end
end

NSOU = int2str(nsources);
NREC = int2str(nreceivers);

tsetup = etime(clock,t00);

% #########################################################################
% #########################################################################
%   
%      Find the paths
%   
% #########################################################################

t00 = clock;
tISEStree = zeros(nsources,1);

if calcpaths == 1,

    if SHOWTEXT >= 1,
        disp(' ')
        disp('####################################################################')
        disp('#')
        disp('#  Finding the paths and computing first-order diffraction')
        disp(' ')
    end
    
    difforder_temporary = 1;
    specorder_temporary = 1;

    % Some variables that are needed for empty combinations

	pathtypevec = [];
	reflpaths = [];
	pathlengthvec = [];
	specextradata = [];
	edgeextradata = [];
	mainlistguide = [];
    Sinsideplanenumber = [];
    Rinsideplanenumber = [];
    mainlistguide = [];
    mainlistguidepattern = [];
    directsoundrow = [];
    allspecrows = [];
    firstdiffrow = [];
    Varlist = [' tfdirect tfgeom tfdiff frequencies Rstart CAIR nedgesubs'];

   if doaddsources == 1
       tfdirect = zeros(nfrequencies,nreceivers);
       tfgeom = zeros(nfrequencies,nreceivers);
       tfdiff = zeros(nfrequencies,nreceivers);        
    elseif doaddsources == 0
       tfdirect = zeros(nfrequencies,nreceivers,nsources);
       tfgeom = zeros(nfrequencies,nreceivers,nsources);
       tfdiff = zeros(nfrequencies,nreceivers,nsources);        
    end
    
    desiredtffile = [Filepath,Filestem,'_tf.mat'];    
    
    if exist('guiderowstouse') ~= 1,
        guiderowstouse = [];    
    end    
    
	if doSbatches == 0,
        eval(['load ''',sdatafile,''''])   
	else
        latestsdatafile = reftoSbatchlist(1);
        [sdatafilepath,filestem,temp1] = fileparts(sdatafile);
        eval(['load ''',[sdatafilepath,filesep,filestem],'_',int2str(latestsdatafile),'.mat'''])     
	end
% % % 	% We create a souinsideplane matrix from the visplanesfroms matrix    
% % % 	% for use in ESIE0makeirsISES
	
	if doRbatches == 0,
        eval(['load ''',rdatafile,''''])
	else
        latestrdatafile = reftoRbatchlist(1);
        [rdatafilepath,filestem,temp1] = fileparts(rdatafile);
        eval(['load ''',[rdatafilepath,filesep,filestem],'_',int2str(latestrdatafile),'.mat'''])     
	end
% % % 	% We create a recinsideplane matrix from the visplanesfromr matrix    
% % % 	% for use in ESIE0makeirsISES

    eval(['load ''',eddatafile,''''])
    
    tISEStree = zeros(nsources,1);
    tmaketfs = zeros(nreceivers,nsources);
    
    for isou = soulist_for_findpaths,
    
        t00 = clock;
        
        ISOU = JJ(isou,1:JJnumbofchars(isou));
        if SHOWTEXT >= 1,
			disp(['Calculating for source ',ISOU,' (of ',NSOU,') '])
		end
        if doSbatches == 1,
            if latestsdatafile ~= reftoSbatchlist(isou),
                latestsdatafile = reftoSbatchlist(isou);
                [sdatafilepath,filestem,temp1] = fileparts(sdatafile);
                eval(['load ''',[sdatafilepath,filesep,filestem],'_',int2str(latestsdatafile),'.mat'''])     
            end
            Scolnumber = isou-Sbatchlist(reftoSbatchlist(isou),1)+1
        else
            Scolnumber = isou;
        end
        S = sources(Scolnumber,:);
        %%%Sdirection = [1 0 0];
	
        %   ###########################################################
        %   #
        %   #   Calculate the ISES tree for each source
        %   #   IS = image sources, for specular reflections
        %   #   ES = edge sources, for edge diffraction
        %   #
        %   ###########################################################

	    if exist('ISEStreefile') ~= 1,
            if SHOWTEXT >= 2,
                disp(['   Calculating ISES tree'])    
            end
            if difforder_temporary >= 1,
                [POTENTIALISES,ORIGINSFROM,ISCOORDS,ISESVISIBILITY,IVNSPECMATRIX,lengthNspecmatrix,IVNDIFFMATRIX,lengthNdiffmatrix, ...
                    singlediffcol,REFLORDER,startindicessinglediff,endindicessinglediff,ndecimaldivider,PointertoIRcombs,IRoriginsfrom] = ...
                    ESIE0findISEStree(planedata,edgedata,edgetoedgedata,S,isou,specorder_temporary,difforder_temporary,Sdata.visplanesfroms(:,isou),Sdata.vispartedgesfroms(:,isou),nedgesubs);
            else
                usedISEStreefile = ESIE0findISEStree(planedata,edgedata,edgetoedgedata,S,isou,specorder_temporary,difforder_temporary,Sdata.visplanesfroms(:,isou),[],nedgesubs);
            end
            
        else
            [ISEStreefilepath,filestem,temp1] = fileparts(ISEStreefile);
            usedISEStreefile = [[ISEStreefilepath,filesep,filestem],'_',ISOU,'_ISEStree.mat;'];   
            if SHOWTEXT >= 2,
                disp(['   Using existing ISEStreefile: ',ISEStreefile,'_',ISOU,'_ISEStree.mat'])    
            end
        end
        tISEStree(isou) = etime(clock,t00);

        if dobackscatter ~= 1, 
            reclist_in_forloop = reclist_for_findpaths;
        else
            reclist_in_forloop = isou;            
        end

        for irec = reclist_in_forloop,
            
            t00 = clock;

            %   ###########################################################
            %   #
            %   #   Find the valid paths for each source-receiver combination
            %   #   _edpaths files are created
            %   #
            %   ###########################################################

            IREC = JJ(irec,1:JJnumbofchars(irec));
        	if SHOWTEXT >= 1,
                if round(irec/idisp)*idisp==irec,  
			        disp(['...receiver no. ',IREC,' (of ',NREC,')'])
                end    
            end
            if doRbatches == 1,
                if latestrdatafile ~= reftoRbatchlist(irec),
                    latestrdatafile = reftoRbatchlist(irec);
                    [rdatafilepath,filestem,temp1] = fileparts(rdatafile);
                    eval(['load ''',[rdatafilepath,filesep,filestem],'_',int2str(latestrdatafile),'.mat'''])     
                end
                Rcolnumber = irec-Rbatchlist(reftoRbatchlist(irec),1)+1;
            else
                Rcolnumber = irec;
            end
            R = receivers(Rcolnumber,:);

            if exist('edpathsfile') ~= 1,
			    desirededpathsfile = [Filepath,Filestem,'_',ISOU,'_',IREC,'_edpaths.mat'];
                if Sdata.soutsidemodel(Scolnumber) == 0 & Rdata.routsidemodel(Rcolnumber) == 0,        
                    if difforder_temporary > 0,
                        [pathtypevec,reflpaths,specextradata,edgeextradata,S,R,mainlistguide,mainlistguidepattern,directsoundrow,allspecrows,firstdiffrow,Sinsideplanenumber,Rinsideplanenumber] = ...
                            ESIE0findpathsISESx(planedata,edgedata,edgetoedgedata,lengthNspecmatrix,lengthNdiffmatrix,singlediffcol,startindicessinglediff,...
                            endindicessinglediff,ndecimaldivider,PointertoIRcombs,IRoriginsfrom,...
                            S,R,directsound,specorder_temporary,difforder_temporary,...
                            nedgesubs,Sdata.visplanesfroms(:,Scolnumber),Rdata.visplanesfromr(:,Rcolnumber),...
                            Sdata.vispartedgesfroms(:,Scolnumber),Rdata.vispartedgesfromr(:,Rcolnumber));

                        [tfdirect_one,tfgeom_one,tfdiff_one] = ESIE0maketfs(planedata,edgedata,edgetoedgedata,pathtypevec,reflpaths,specextradata,edgeextradata,S,R,...
                            mainlistguide,mainlistguidepattern,directsoundrow,allspecrows,firstdiffrow,Sinsideplanenumber,Rinsideplanenumber,specorder_temporary,frequencies,Rstart,...                        
                            guiderowstouse,directsound);
                        
                        if doaddsources == 1
                            tfdirect(:,irec) = tfdirect(:,irec) + tfdirect_one; 
                            tfgeom(:,irec)   = tfgeom(:,irec)   + tfgeom_one; 
                            tfdiff(:,irec)   = tfdiff(:,irec)   + tfdiff_one; 
                        else
                            tfdirect(:,irec,isou) = tfdirect(:,irec,isou) + tfdirect_one; 
                            tfgeom(:,irec,isou)   = tfgeom(:,irec,isou)   + tfgeom_one; 
                            tfdiff(:,irec,isou)   = tfdiff(:,irec,isou)   + tfdiff_one; 
                        end
                     
                    else

                        error('ERROR: Havent fixed this yet!')
                        % Should it really be Scolnumber/Rcolnumber
                        % below??? They will always have the value 1 if
                        % nRbatches = nreceivers??
                        
%                         usededpathsfile = ESIE0findpathsISESx(eddatafile,lengthNspecmatrix,[],[],[],...
%                             [],[],[],[],S,R,Scolnumber,Rcolnumber,directsound,specorder_temporary,difforder_temporary,...
%                             nedgesubs,visplanesfroms(:,Scolnumber),visplanesfromr(:,Rcolnumber),[],[],desirededpathsfile);
                    end
                else
%                     usededpathsfile = desirededpathsfile;
%                     eval(['save ',usededpathsfile,Varlist])
                end
			else
				if SHOWTEXT >= 2,
					disp(['   Using existing edpathsfile: ',edpathsfile,'_',ISOU,'_',IREC,'_edpaths.mat'])
                    desirededpathsfile = [edpathsfile,'_',ISOU,'_',IREC,'_edpaths.mat'];
				end
            end
            
            tmaketfs(irec,isou) = etime(clock,t00);
	
		end		% of the for irec = ,
	
	end		% of the for isou = ,
    
    eval(['save ''',desiredtffile,'''',Varlist])   
    
end

% #########################################################################
% #########################################################################
%   
%      Calculate frequency-domain results using the integral equation
%      method.
%   
% #########################################################################

if calcinteq == 1,
    
    eval(['load ''',eddatafile,''''])
    if exist('doaddsources') ~= 1
       doaddsources = 0; 
    end
    if difforder < 2,
        disp(' ')
        error(['ERROR: The integral equation can only be used for difforder >= 2.'])             
    end

    if SHOWTEXT >= 1,
        disp(' ')
        disp('####################################################################')
        disp('#')
        disp('#  Calculating frequency-domain results (TFs)')
        disp('#  using the integral equation method')
        disp(' ')
    end
    
    % If the parameter guiderowstouse wasn't specified in the setup file,
    % we give it the default value.
    
    if exist('guiderowstouse') ~= 1,
        guiderowstouse = [];    
    end

    % Some variables that are needed for empty combinations
    tfinteqgeom = [];
    tfinteqtot = [];
    tfinteqdirect = [];
    tfinteqdiff = [];
    P_atedges = [];
    if dobackscatter == 1
        Varlist = [' tfinteqdirect tfinteqgeom tfinteqdiff tfinteqtot frequencies Rstart CAIR nedgesubs inteq_solmethod inteq_niter inteq_ngauss P_atedges'];
    else
        Varlist = [' tfinteqdirect tfinteqgeom tfinteqdiff tfinteqtot frequencies Rstart CAIR nedgesubs inteq_solmethod inteq_niter inteq_ngauss'];        
    end


    
    % First, the edge-to-edge submatrix structure is developed because it
    % is independent of source position, receiver position, edge element
    % size and frequency.
    t00 = clock;
    [Hsubmatrix,listofsubmatrices,edgepairlist,edgetripletlist,submatrixcounter,nedgeelems,bigmatrixstartnums,bigmatrixendnums,reftoshortlist,isthinplanetriplet,isthinplaneedgepair,...
        quadraturematrix_pos,quadraturematrix_weights] = ...
        ESIE0integ_e2estructure(edgedata.edgelengthvec,edgedata.closwedangvec,inteq_ngauss,inteq_discretizationtype,edgetoedgedata,edgedata.planesatedge);
    tESSIEstructure = etime(clock,t00);

    if SHOWTEXT >= 1
        disp(['Number of edges = ',int2str(nedges)])
        disp(['Total number of edge elements = ',int2str(sum(nedgeelems))])
        disp(['Number of edge source signals to compute = ',int2str(max(bigmatrixendnums))])
    end    
    
    [tfinteqdiff,tESSIEinitial,tESSIEmatrix,tESSIEmatrixsubtimes,tESSIEsourceterm,tESSIEedgesignals,tESSIEpropagate] = ESIE0integralequation(Filepath,Filestem,planedata,edgedata,edgetoedgedata,Hsubmatrix,listofsubmatrices,edgepairlist,edgetripletlist,submatrixcounter,nedgeelems,bigmatrixstartnums,bigmatrixendnums,reftoshortlist,isthinplanetriplet,isthinplaneedgepair,...
        quadraturematrix_pos,quadraturematrix_weights,Sdata,doaddsources,Rdata,inteq_ngauss,inteq_discretizationtype,frequencies,...
        inteq_solmethod,inteq_niter,inteq_savesourcesignals,Rstart);   
    
    desiredtfinteqfile = [Filepath,Filestem,'_tfinteq.mat'];
    eval(['save ''',desiredtfinteqfile,'''',Varlist])   
    
end
    

% #########################################################################
% #########################################################################
%   
%      Save the timing data
%   
% #########################################################################

logfilename = [Filepath,Filestem,'_log.mat'];
Varlist = [' tcadgeo tedgeo tsgeo trgeo ted2geo tsetup tISEStree tmaketfs tESSIEstructure tESSIEinitial tESSIEmatrix tESSIEmatrixsubtimes tESSIEsourceterm tESSIEedgesignals tESSIEpropagate'];
eval(['save ''',logfilename,'''',Varlist])
