function [tfdirect,tfgeom,tfdiff] = ESIE0maketfs(planedata,edgedata,edgetoedgedata,...
    pathtypevec,reflpaths,specextradata,edgeextradata,S,R,mainlistguide,mainlistguidepattern,...
    directsoundrow,allspecrows,firstdiffrow,Sinsideplanenumber,Rinsideplanenumber,specorder,...
    frequencies,Rstart,guiderowstouse,includedirectsound)
% ESIE0maketfs - Constructs transfer functions from a list of paths.
%
% Input parameters:
%   planedata           struct
%   edgedata            struct
%   edgetoedgedata      struct
%   pathtypevec,reflpaths,specextradata,edgeextradata,S,R, ...
%   mainlistguide,mainlistguidepattern,directsoundrow,allspecrows, ...
%   firstdiffrow,Sinsideplanenumber,Rinsideplanenumber
%                       Data generated by the function ESIE0findpathsISESx
% 	specorder           The maximum order of reflections that is calculated.
%   frequencies         A list of frequencies that the TF should be
%                       computed for.
% 	Rstart              The reference distance that the distance zero of
%                       the transfer response refers to.
%   guiderowstouse      (optional) A list of values, indicating which rows in the
%                       mainlistguide and mainlistguidepattern that should
%                       be used. This way it is possible to select only
%                       diffraction, for instance. If this list is not
%                       specified, all components will be used.
%   includedirectsound  (optional) 0 or 1, indicating whether the direct
%                       sound should be included or not. Default: 1 
%   CAIR, SHOWTEXT   Global parameters.
%
% Output parameters:
%   tfdirect            The TF containing the direct sound, size [nfrequencies,1].
%   tfgeom              The TF containing the specular reflections, size [nfrequencies,1].
%   tfdiff              The TF containing the diffracted components, size [nfrequencies,1].
%
% Uses functions ESIE0calcdist  ESIE0coordtrans2 ESIE0wedge1st_fd ESIE0findis 
%
% ----------------------------------------------------------------------------------------------
%   This file is part of the Edge Diffraction Toolbox by Peter Svensson.                       
%                                                                                              
%   The Edge Diffraction Toolbox is free software: you can redistribute it and/or modify       
%   it under the terms of the GNU General Public License as published by the Free Software     
%   Foundation, either version 3 of the License, or (at your option) any later version.        
%                                                                                              
%   The Edge Diffraction Toolbox is distributed in the hope that it will be useful,       
%   but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS  
%   FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.             
%                                                                                              
%   You should have received a copy of the GNU General Public License along with the           
%   Edge Diffraction Toolbox. If not, see <http://www.gnu.org/licenses/>.                 
% ----------------------------------------------------------------------------------------------
% Peter Svensson 31 March 2015 (svensson@iet.ntnu.no)
%
% [tfdirect,tfgeom,tfdiff] = ESIE0maketfs(planedata,edgedata,edgetoedgedata,...
%     pathtypevec,reflpaths,specextradata,edgeextradata,S,R,mainlistguide,mainlistguidepattern,...
%     directsoundrow,allspecrows,firstdiffrow,Sinsideplanenumber,Rinsideplanenumber,specorder,...
%     frequencies,Rstart,guiderowstouse,includedirectsound);

% 16 Jul 2010   Functioning version
% 3 Dec 2014    Changed to using structs for input
% 31 March 2015 Cleaned up the help text and removed some redundant input
%               parameters.

global SHOWTEXT CAIR

%--------------------------------------------------------------------------
% We look for the optional vector multfactors .
% If it was there, then those values will be used to switch off, or boost,
% any reflection path.
% Those values will only be used for diffraction paths.

if exist('multfactors') == 0,
    multfactors = ones(size(reflpaths,1),1);    
else
    if size(multfactors,1) ~= size(reflpaths,1),
        error(['The edpaths file contained a vector multfactors which does not have the same length as reflpaths.'])    
    end
end

%--------------------------------------------------------------------------

Sdirection = [1 0 0];
maxnedgesorplanes = max(max(reflpaths));
if ~isempty(maxnedgesorplanes),
    multfac = 10^ceil(log10(double(maxnedgesorplanes)+1));
else
    multfac = 0;
end

if nargin < 21,
    includedirectsound = 1;
end
if nargin < 20,
    guiderowstouse = [];
end
if isempty(guiderowstouse),
    usesubset = 0;    
    if SHOWTEXT > 2,
        disp('Using all components')
    end
else
    usesubset = 1;        
    if SHOWTEXT > 2,
        disp('Using only some components')
    end
end

nyvec = pi./(2*pi - edgedata.closwedangvec);    
onesvec1 = ones(1,3);

souspecboost = 1;
if ~isempty(Sinsideplanenumber),
    if reflfactors(Sinsideplanenumber(1)) ~= 0,
        souspecboost = 2;
    end
end

recspecboost = 1;
if ~isempty(Rinsideplanenumber),
    if reflfactors(Rinsideplanenumber(1)) ~= 0,
        recspecboost = 2;
    end
end

if isempty(edgetoedgedata.re1sho),
    userwantsdiff2 = 0;
else
    userwantsdiff2 = 1;    
end

nfrequencies = length(frequencies);

%-------------------------------------------------------

if exist('mainlistguide') ~= 1,
    mainlistguide = [];    
else
    mainlistguide = double(mainlistguide);    
end

[ncomponents,ncols] = size(reflpaths);
nrowsinlist = size(mainlistguide,1);
if ncols > 1,
    difforderinlist = sum(mainlistguidepattern.'=='d').';
else
    difforderinlist = (mainlistguidepattern=='d');    
end

lastNdiffrow = zeros(1,specorder);
for ii = 1:specorder,
    iv = find(difforderinlist <= ii );
    if ~isempty(iv),
        lastNdiffrow(ii) = iv(end);
    end
end

nrefltogetherwdiff = sum(mainlistguidepattern.'=='d').'.*(  sum(mainlistguidepattern.'=='s').');

if SHOWTEXT >= 3,
	disp(['   Constructing TF. ',int2str(ncomponents),' components.'])
end

tfdirect = zeros(nfrequencies,1);
tfgeom   = zeros(nfrequencies,1);
tfdiff   = zeros(nfrequencies,1);
tftot    = zeros(nfrequencies,1);

% % Varlist = [' tfdirect tfgeom tfdiff tftot frequencies Rstart CAIR EDcalcmethod'];

if ncomponents == 0,
    return
end

%##############################################################################
%##############################################################################
%##############################################################################
%##############################################################################
%
%  Diffraction once, possibly with pre- and post-specular reflections
%
%##############################################################################

directsoundonboundary = 0;
specreflonboundary = zeros(size(planedata.planeeqs,1),1);

if firstdiffrow ~= 0,
    
    % First we remove the 'd' for all the rows in the mainlistguidepattern that
    % we do not want to use.
    
    if usesubset == 1,
        singdiffrows = find(difforderinlist==1);
        rowstoclear = setdiff(singdiffrows,guiderowstouse);
        if ~isempty(rowstoclear),
            mainlistguidepattern(rowstoclear,:) = mainlistguidepattern(rowstoclear,:)*0;    
        end
    end

    % Here we can use the single-diffraction calculation for all
    % combinations. The source should either be the original source or
    % an image source. The receiver should either be the original
    % receiver or an image receiver.
    
    % First we create lists that specify whether there are specular
    % reflections before the edge diffraction, and after the edge
    % diffraction. Also, a list which gives the column where the edge
    % diffraction is present can be used to efficiently extract the
    % edge number. 
    % NB! The lists prespeclist, postspeclist and singdiffcol all have
    % the (short) length of the mainlistguide. 
    % The data in these lists will then be used to create the long
    % lists prespecpresent and postspecpresent which simply are 0 or 1.
    % A matrix called edgemask will have the same number of columns as
    % the reflpaths matrix. For each row, there will be a 1 in the
    % column where the edge diffraction is so we can efficiently find
    % the edge number.

    multmatrix = [1:specorder];
    if size(mainlistguidepattern,2) > specorder,
        multmatrix = [1:size(mainlistguidepattern,2)];
    end
    multmatrix = multmatrix( ones(nrowsinlist,1),:  );
    
    if ncols > 1,
        singdiffcol = sum( (multmatrix.*(mainlistguidepattern=='d')).' ).';
        singdiffcol = singdiffcol.*(sum(mainlistguidepattern.'=='d').'<=1);
        nrefl = sum( (mainlistguidepattern=='s' | mainlistguidepattern=='d').' ).';
    else
        singdiffcol = mainlistguidepattern=='d';
        nrefl = singdiffcol | mainlistguidepattern=='s';
    end
    
    prespeclist = singdiffcol-1;
    prespeclist = prespeclist.*(prespeclist>0);
    postspeclist = (nrefl-singdiffcol).*(singdiffcol~=0);
    
    prespecpresent = zeros(ncomponents,1);
    postspecpresent = zeros(ncomponents,1);
    edgemask = zeros(ncomponents,specorder);

    diffrowsthatareOK = [firstdiffrow:lastNdiffrow(1)];
    if usesubset == 1,
        diffrowsthatareOK = intersect(diffrowsthatareOK,guiderowstouse);
    end
    for ii = diffrowsthatareOK,    %firstdiffrow:lastNdiffrow(1),
        iv = [(mainlistguide(ii,2)):(mainlistguide(ii,3))];
        onesvec2 = ones(length(iv),1);
        edgemask(iv,singdiffcol(ii)) = onesvec2;            
%         if ii > specorder+2,
            prespecpresent(iv) = (prespeclist(ii)>0)*onesvec2;
            postspecpresent(iv) = (postspeclist(ii)>0)*onesvec2;
%         end
    end
    prespecpresent = prespecpresent(:,onesvec1);
    postspecpresent = postspecpresent(:,onesvec1);

    if ncols > 1,
%         edgenumb = sum( (edgemask.*double(reflpaths)).').';
        edgenumb = sum( (edgemask.*double(reflpaths(:,1:specorder))).').';
    else
        edgenumb = edgemask.*double(reflpaths);    
    end

    nedgetfs = size(edgenumb,1);
    nnonzeroedgetfs = sum(edgenumb(:,1)>0);

    tfcounter = 0;
    edgelist = unique(edgenumb);
    for ii = 1: length(edgelist),
        edge = edgelist(ii);
        if edge~= 0,
           iv = find(edgenumb==edge);
           ncombs = length(iv);
           
           if ncombs > 0 & any(multfactors(iv)),
               if SHOWTEXT >= 4,
                    disp(['   Edge ',int2str(edge),': ',int2str(ncombs),' combinations'])    
               end               
                 
                onesvec3 = ones(ncombs,1);
	
                IS = full(specextradata(iv,1:3));
                IR = full(specextradata(iv,4:6));                
            
                edgestart = full(edgeextradata(iv,1));
                edgeend   = full(edgeextradata(iv,2));
                
                % Calculate the cyl coordinates of all IS
                % 
                % Calculate the cyl coord of all IR
	
                edgecoords = [edgedata.edgestartcoords(edge,:);edgedata.edgeendcoords(edge,:)];
            	[rs,thetas,zs,rr,thetar,zr] = ESIE0coordtrans2(IS,IR,edgecoords,edgedata.edgenvecs(edge,:));
% % % % %             	[rs,thetas,zs,rr,thetar,zr] = ESIE0coordtrans2(full(specextradata(iv,1:3)),full(specextradata(iv,4:6)),edgecoords,edgenvecs(edge,:));
	
				bc = planedata.reflfactors(edgedata.planesatedge(edge,:)).';
% 				if prod(double(bc==[1 1])) ~= 1,
% 					disp('   Non-rigid wedge')
% 				end
	
                for jj = 1:ncombs,
                    if multfactors(iv(jj)) > 0,

                       [tfnew,singularterm] = ESIE0wedge1st_fd(frequencies,edgedata.closwedangvec(edge),rs(jj),thetas(jj),zs(jj),rr(jj),thetar(jj),zr(jj),...
                         edgedata.edgelengthvec(edge)*[edgestart(jj) edgeend(jj)],'n',Rstart,bc);                  

                          if any(singularterm),
                              if singularterm(2) | singularterm(3),
                                  directsoundonboundary = 1;
                            elseif singularterm(1),
                                if specorder == 1,
                                   specreflonboundary(planesatedge(edge,2)) = 1;
                               else
                                   disp('WARNING! A specular refl. of order > 1 is half obscured but this is not handled yet!');    
                               end
                            elseif singularterm(4),
                                  if specorder == 1,
                                       specreflonboundary(planesatedge(edge,1)) = 1;
                                   else
                                       disp('WARNING! A specular refl. of order > 1 is half obscured but this is not handled yet!');    
                                   end
                            end
                          end
		
                    
                        % Decide whether the TF should be boosted or not
                        %
                        % The factors souspecboost and recspecboost have
                        % values other than 1 if the source or receiver is directly
                        % at a plane, for the boosting of the direct sound, and
                        % specular reflections.
                        %
                        % This boost factor should be used also for edge
                        % diffraction if:
                        %   1.  There are some specular reflections between the 
                        %       source and the edge
                        %   or
                        %   2.  There are no specular reflections between the
                        %       source and the edge, but the source/receiver is at
                        %       a plane which does not belong to the edge.
                        
                        if souspecboost ~= 1 | recspecboost ~= 1,
                        
                            boostfactor = 1;
                            if prespecpresent(iv(jj)) == 1,
                                boostfactor = souspecboost;    
                            else
                                if ~isempty(Sinsideplanenumber),
                                    if Sinsideplanenumber(1)~=planesatedge(edge,1) & Sinsideplanenumber(1)~=planesatedge(edge,2), 
                                        boostfactor = souspecboost;
                                    end
                                end
                            end
                            if postspecpresent(iv(jj)) == 1,
                                boostfactor = boostfactor*recspecboost;    
                            else
                                if ~isempty(Rinsideplanenumber),
                                    if Rinsideplanenumber(1)~=planesatedge(edge,1) & Rinsideplanenumber(1)~=planesatedge(edge,2), 
                                        boostfactor = boostfactor*recspecboost;
                                    end
                                end
                            end
                            if boostfactor ~= 1,
                                tfnew = tfnew*boostfactor;    
                            end
                        
                        end    % ....    if souspecboost ~= 1 | recspecboost ~= 1,


                        tfdiff = tfdiff + tfnew*double(multfactors(iv(jj)));



                    end   %  ...  if multfactors(iv) > 0,
                    
                end  % .....                for jj = 1:ncombs,

            end   %....    if ncombs > 0 & any(multfactors(iv)),
            
        end    % ....         if edge~= 0,
        
    end  % ....     for ii = 1: length(edgelist),
    
end   % ...if firstdiffrow ~= 0,

nspecreflonboundary = sum(specreflonboundary);


%##############################################################################
%##############################################################################
%##############################################################################
%##############################################################################
%
%      The direct sound
%
%##############################################################################

if usesubset == 1 & directsoundrow == 1,
    if any(guiderowstouse == 1) == 0,
        directsoundrow = 0;    
    end
end

if directsoundrow == 1 & includedirectsound == 1,
    dist = norm(R-S);
%     slotnumberfrac = (dist-Rstart)/CAIR*FSAMP+1;
    amp = souspecboost*recspecboost./dist;
    if directsoundonboundary,
        amp = amp/2;    
        if SHOWTEXT >= 4,
            disp('HALVING DIRECT SOUND')
        end
    end
    tfdirect = amp*exp(-j*2*pi*frequencies(:)/CAIR*(dist-Rstart));
    
	if ncomponents == 1,
        tftot = tftot + tfdirect;
        
        return
	end    
else
    if directsoundonboundary == 1,
        
        dist = norm(R-S);
%         slotnumberfrac = (dist-Rstart)/CAIR*FSAMP+1;
        amp = souspecboost*recspecboost./dist;
        if directsoundonboundary,
            amp = amp/2;    
            if SHOWTEXT >= 4,
                disp('HALVING DIRECT SOUND')
            end
        end
        tfdirect = amp*exp(-j*2*pi*frequencies(:)/CAIR*(dist-Rstart));
        
      
    end
    if SHOWTEXT >= 4,
        disp(['      No direct sound'])    
    end
end

%##############################################################################
%##############################################################################
%##############################################################################
%##############################################################################
%
%  All-specular			s, ss, sss, etc
%
%##############################################################################

if allspecrows(1) ~= 0,
    if usesubset == 1,
        specrowsthatareOK = intersect(allspecrows,guiderowstouse);
    else
        specrowsthatareOK = [allspecrows(1):allspecrows(2)];
    end
    if ~isempty(specrowsthatareOK),
        if usesubset == 1,
            temp = mainlistguide(specrowsthatareOK,2:3);
            ivspec = [double(mainlistguide(specrowsthatareOK(1),2)):double(mainlistguide(specrowsthatareOK(1),3))];
            for ii = 2:size(temp,1);
                ivspec = [ivspec  [double(mainlistguide(specrowsthatareOK(ii),2)):double(mainlistguide(specrowsthatareOK(ii),3))]];
            end
        else
            ivspec = [mainlistguide(allspecrows(1),2):mainlistguide(allspecrows(2),3)];        
        end
        
        if nspecreflonboundary > 0,
            listofspecreflonboundary = find(specreflonboundary);
            specrefltoadd = setdiff(listofspecreflonboundary,ivspec)
            if ~isempty(specrefltoadd),
                error(['ERROR: We need to add some code here, to reinsert pruned spec. refl.'])    
            end
        end
        
        if SHOWTEXT >= 4,
            disp(['      ',int2str(length(ivspec)),' specular reflections'])    
        end
        dist = ESIE0calcdist(full(specextradata(ivspec,1:3)),R);
        if size(reflpaths,2) == 1
            refl = planedata.reflfactors(  reflpaths(ivspec) );
        else
        	if SHOWTEXT >= 1
	           disp('WARNING: havent implemented reflfactors yet for spec reflorder >1!'); 
    	       disp('   reflfactors will be set to 1');
    	   end
           refl = 1;
        end
        
        nspec = length(dist);

        if nspecreflonboundary > 0,
            specamp = ones(size(souspecboost));
            amplitudeshouldbehalf = ismember(reflpaths(ivspec,1),listofspecreflonboundary);
            specamp = specamp - amplitudeshouldbehalf*0.5;
            if nspec == 1
                tfnew = specamp*souspecboost*recspecboost*refl.*exp( -j*2*pi*frequencies(:)/CAIR*(dist-Rstart))/dist;		               
            else
                dist = dist(:).';
                frequencies = frequencies(:);
               tfnew = specamp*souspecboost*recspecboost*refl.*exp( -j*2*pi*frequencies(:,ones(1,nspec))/CAIR.*(dist(ones(nfrequencies,1),:)-Rstart))./dist(ones(nfrequencies,1),:);	 
               tfnew = sum(tfnew.').';
            end
        else
%     			irnew = souspecboost*recspecboost*ESIE0irfromslotlist((dist-Rstart)/CAIR*FSAMP+1,1./dist);		

            if nspec == 1
                tfnew = souspecboost*recspecboost*refl.*exp( -j*2*pi*frequencies(:)/CAIR*(dist-Rstart))/dist;		               
            else
                dist = dist(:).';
                refl = refl(:).';
                frequencies = frequencies(:);
%                    tfnew = souspecboost*recspecboost*refl.*exp( -j*2*pi*frequencies(:,ones(1,nspec))/CAIR.*(dist(ones(nfrequencies,1),:)-Rstart))./dist(ones(nfrequencies,1),:);	 
               tfnew = souspecboost*recspecboost*refl(ones(nfrequencies,1),:).*exp( -j*2*pi*frequencies(:,ones(1,nspec))/CAIR.*(dist(ones(nfrequencies,1),:)-Rstart))./dist(ones(nfrequencies,1),:);	 
               tfnew = sum(tfnew.').';
            end
        end
        tfgeom = tfgeom + tfnew;
            
    else
        if nspecreflonboundary > 0,
            listofspecreflonboundary = find(specreflonboundary);
            error(['ERROR: We need to add some code here, to reinsert pruned spec. refl., pos. 2'])    
        end        
    end    
else
% % %     if nspecreflonboundary > 0,
% % %         listofspecreflonboundary = find(specreflonboundary);
% % %         
% % %         [xis] = ESIE0findis(S,listofspecreflonboundary,planeeqs,1,[1 1 1]);
% % %         
% % %         disp(['WARNING: We have some new code here, to reinsert pruned spec. refl., pos. 3'])
% % %         dist = ESIE0calcdist(xis,R);
% % %         irnew = 0.5*souspecboost*recspecboost*ESIE0irfromslotlist((dist-Rstart)/CAIR*FSAMP+1,1./dist);		                
% % %         ngeom = length(irgeom);
% % %         nnew = length(irnew);
% % %         if nnew > ngeom,
% % %            irgeom = [irgeom;zeros(nnew-ngeom,1)];
% % %         end
% % %         irgeom(1:nnew) = irgeom(1:nnew) + irnew;
% % %         
% % %     end        
end

%##############################################################################

tftot = tftot + tfdirect;

tftot = tftot + tfgeom;

tftot = tftot + tfdiff;

